<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * @file controller/admin_template.php
 * Parent class for all Admin controllers. Implements HTTP Digest Authentication.
 * @author Michael Oberg
 */
class admin_template {
    /*
     * Construction of an object of an derived class will force HTTP Digest
     * Authentication against the admin credentials stored in the database.
     * An Exception is thrown if authentication fails.
     */
    public function __construct() {
        $c = ajaxsyncDB::selectSingle("select digest from credentials limit 1");
        if (!$c["digest"])
            throw new Exception("Failed: no digest stored for admin");
        $httpDigest = new httpDigestAuth("AjaxSync Administration", $c["digest"]);
        // raise password dialog if necessary; kill script with exception if not
        // authenticated:
        $httpDigest->checkHttpAuth();
    }
    /*
     * Every class derived from admin_template has to implement a run method.
     */
    public function run($request) {
        throw new Exception("Failed: admin_template::run has to be overloaded.");
    }
}
?>