<?php
/****************************************************************************
 * Copyright (C) 2011  Michael Oberg
 * 
 * This file is part of the AjaxSync chat application.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * @file controller/authDigest.php
 * Parent class for all AjaxSync chat controllers except Admin controllers. Implements
 * HTTP Digest Authentication.
 * @author Michael Oberg
 */
class authDigest {
    /*
     * Construction of an object of an derived class will force HTTP Digest
     * Authentication against the credentials stored in the database.
     * An Exception is thrown if authentication fails.
     */
    public function __construct() {
        if (!isset($_SERVER['PHP_AUTH_DIGEST'])) {
            $this->sendHeader();
            throw new Exception("Failed: Not authenticated");
        }
        if (!($digestdata = $this->http_digest_parse($_SERVER['PHP_AUTH_DIGEST']))) {
            $this->sendHeader();
            throw new Exception("Failed: Wrong credentials");
	}
	$this->user = $digestdata['username'];
        $username = mysql_escape_string($this->user);

        $sql = "select digest from users where name = '$username' and status = 2";
        $c = ajaxsyncDB::selectSingle($sql);
        if ($c) {
            $A1 = $c["digest"];
        }
        else {
            $this->sendHeader();
            throw new Exception("Failed: Wrong credentials (User unknown)");
        }
        $A2 = md5($_SERVER['REQUEST_METHOD'] . ':' . $digestdata['uri']);
        $answer = md5($A1 . ':' . $digestdata['nonce'] . ':' . $digestdata['nc'] .
            ':' . $digestdata['cnonce'] . ':' . $digestdata['qop'] . ':' .
            $A2);
        if ($digestdata['response'] != $answer) {
            $this->sendHeader();
            throw new Exception("Failed: Wrong credentials");
        }
    }

    public function logout() {
        $this->sendHeader();
    }

    private function sendHeader() {
        header('HTTP/1.1 401 Unauthorized');
        header('WWW-Authenticate: Digest realm="' . AJAXSYNC_CHAT_REALM .
            '",qop="auth",nonce="' . uniqid() . '",opaque="' . md5(AJAXSYNC_CHAT_REALM) .
            '"');
    }
 
    private function http_digest_parse($txt) {
        $usage = array('nonce'=>1, 'nc'=>1, 'cnonce'=>1, 'qop'=>1,
            'username'=>1, 'uri'=>1, 'response'=>1);
        $digestdata = array();
        $keys = implode('|', array_keys($usage));
        preg_match_all('@(' . $keys . ')=(?:([\'"])([^\2]+?)\2|([^\s,]+))@',
            $txt, $matches, PREG_SET_ORDER);
        foreach ($matches as $t) {
            $digestdata[$t[1]] = $t[3] ? $t[3] : $t[4];
            unset($usage[$t[1]]);
        }
        return $usage ? false : $digestdata;
    }
 
    /*
     * Every class derived from authDigest has to implement a run method.
     */
    public function run($request) {
        throw new Exception("Failed: authDigest::run has to be overloaded.");
    }

    /*
     * Get the username of the running session.
     */
    public function getUsername() {
        return $this->user;
    }

    /*
     * Storage for the username.
     */
    private $user;
}
?>
