<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 *
 * This file is part of the AjaxSync framework.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * Controller / AjaxSync HTTP function call. Initializes or continues/recovers
 * an AjaxSync session.
 *
 * @author Michael Oberg
 */
class initReset extends authDigest {
    public function run($request) {
        try {
            ajaxsyncSession::session_start();
        }
        catch (Exception $e) {
            echo $e->getMessage();
            return;
        }
        // initReset differs from init: no test of an open session ...
        if ((ajaxsyncSession::$values["AJAXSYNC_USER"] == $this->getUsername())
            && ($user = ajaxsyncSession::$values["AJAXSYNC_USER"])) {
            // Synchronizer session is expired or was never started
            if ($errmsg = ajaxsyncCore::startSession()) {
                // a Synchronizer error occured
                echo $errmsg;
                return;
            }
            if ($loginMsg = ajaxsyncAccessControl::checkLogin()) {
                // session was already closed in checkLogin, and all listeners were removed
                ajaxsyncSession::$values["AJAXSYNC_CID"] = 0;
                echo $loginMsg . "Last call was: 0";
            }
            else {
                // success: close and update the session with new timestamp;
                // with the first wait call all listeners will be re-registered
                echo "OK: logged in as $user\nLast call was: "
                        . ajaxsyncSession::$values["AJAXSYNC_CID"];
            }
            ajaxsyncSession::session_write_close();
            return;
        }
        if ($errmsg = ajaxsyncCore::startSession()) {
            // a Synchronizer error occured
            echo $errmsg;
            return;
        }
        // cleanup in case the session object was not correctly updated after logout/timeout
        ajaxsyncBuffer::unregisterAll();
        ajaxsyncCore::removeAllListeners();
        ajaxsyncSession::$values["AJAXSYNC_CID"] = 0;
        ajaxsyncSession::session_write_close();
        echo "OK: not logged in\nLast call was: 0";
        return;
    }
}
?>
