<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 *
 * This file is part of the AjaxSync framework.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * Controller / AjaxSync HTTP function call. Proceed a logout/login action for
 * an AjaxSync session. Gets username, password and a list of function handlers
 * to register as parameters. Returns the login success state and in case of a
 * successful login the registration success state for each function handler.
 *
 * @author Michael Oberg
 */
class login extends authDigest {
    public function run($request) {
        // open the session object - it's required from ajaxsyncBuffer
        ajaxsyncSession::session_start();
        if (!ajaxsyncSession::getSID()) {
            echo "Failed: Session is not open";
            return;
        }
        // Old code:
        // if (ajaxsyncAccessControl::login($request["user"], $request["password"])) {
        // New code:
        // With using authDigest, we will not store the password in the database, this DB
        // field will be empty; and the username is given by the digest credentials. So
        // we are calling the classic authentication using the digest username and an empty
        // password.
        if (($request["user"] == $this->getUsername()) && ajaxsyncAccessControl::login($this->getUsername(), "")) {
            // last access time and timeout already stored by ajaxsyncAccessControl::login
            $success = array();
            $handler = explode("\n",$request["handler"]);
            foreach($handler as $h) {
                if ($h) { // last element is always empty
                    $listener = ajaxsyncDataExchange::readList($h);
                    if (ajaxsyncBuffer::register($listener["fname"])) {
                        if ($err = ajaxsyncCore::listenerRegistration($listener["fname"])) {
                            $success[$listener["fname"]] = 0;
			    ajaxsyncLogger::writeMessage($err);
                        }
			else {
                            $success[$listener["fname"]] = 1;
                        }
                    }
                    else {
                        $success[$listener["fname"]] = 0;
                    }
                }
            }
            ajaxsyncSession::session_write_close();
            // Old code:
            // echo "OK: logged in as " . $request["user"] . "\n"
            // New code (with authDigest):
            echo "OK: logged in as " . $this->getUsername() . "\n"
                    . ajaxsyncDataExchange::writeList($success) . "\n";
            return;
        }
        else {
            // listener registrations already are removed by ajaxsyncAccessControl::login
            ajaxsyncSession::session_write_close();
            echo "OK: not logged in";
            return;
        }
    }
}
?>
