<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * Controller / AjaxSync HTTP function call. Puts function calls from the clients
 * buffer into the servers buffer and sends the appropriate signals. Returns the
 * success state for each call.
 *
 * @author Michael Oberg
 */
class send extends authDigest {
    public function run($request) {
        // open the session object - it's required from ajaxsyncBuffer
        ajaxsyncSession::session_start();
        // write OK message if not logged in, ignore call but confirm
        if (ajaxsyncSession::$values["AJAXSYNC_USER"] != $this->getUsername()) {
            $loginMsg = "OK: not logged in"; // should not happen - username changed due to relogin
            echo $loginMsg;
        }
        else if ($loginMsg = ajaxsyncAccessControl::checkLogin()) {
            echo $loginMsg;
        }
        // WARNING: when the client escapes data we will receive them unescaped,
        // there is no need to run urldecode. Especially for data handled by
        // ajaxsyncDataExchange a manual decode would be disastrous ...
        $calls = explode("\n",$request["calls"]);
        foreach($calls as $c) {
            if ($c) { // last element is always empty
                $call = ajaxsyncDataExchange::readList($c);
                if ($loginMsg) {
                    $ret[$call["cid"]] = 1; // confirm, but do not send
                }
                else { // try to send
                    // First: push the call in the ajaxsyncBuffer, if possible, and store the success/confirmation message
                    if ($ret[$call["cid"]] = ajaxsyncBuffer::makeCall($call["cid"], $call["fname"], $call["param"], $call["timeout"])) {
                        // and then send the signal
                        if ($errmsg = ajaxsyncCore::kill($call["fname"])) {
                            echo $errmsg;
                        }
                    }
                }
            }
        }
        // close and update the session
        ajaxsyncSession::session_write_close();
        // Return the success status for each call
        echo ajaxsyncDataExchange::writeList($ret) . "\n";
    }
}
?>
