<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * Controller / AjaxSync HTTP function call. Waits for a signal and then returns
 * all function calls in the buffer.
 *
 * @author Michael Oberg
 */
class wait extends authDigest {
    public function run($request) {
        // open the session object - it's required from ajaxsyncBuffer
        ajaxsyncSession::session_start();
        if (ajaxsyncSession::$values["AJAXSYNC_USER"] != $this->getUsername()) {
            $loginStatus = false; // should not happen - username changed due to relogin
        }
        else if (!ajaxsyncAccessControl::checkLogin()) {
            $loginStatus = true;
            ajaxsyncDB::change(
                "update users set lastaccess = now() where id = "
                . ajaxsyncSession::$values["AJAXSYNC_UID"]
            ); // set timestamp
        }
        else {
            $loginStatus = false; // login timeout or explicite logout
        }
        // First, get the success status from the previous run of "wait".
        // WARNING: when the client escapes data we will receive them unescaped,
        // there is no need to run urldecode. Especially for data handled by
        // ajaxsyncDataExchange a manual decode would be disastrous ...
        if ($request["ret"]) { // test if we have something to confirm
            $ret = ajaxsyncDataExchange::readList($request["ret"]);
            foreach($ret as $key => $value) {
                // $value should be 0 or 1 (confirmed or not), $key should
                // always be defined. However, due to a Connector error that
                // was not the case, so we test both key and value.
                if ($key && $value) {
                    ajaxsyncBuffer::confirmCall($key);
                }
            }
        }
        // update the session before calling wait, it will be reopened after wait
        ajaxsyncSession::session_write_close();
        // Now look if there are already calls waiting for us
        $calls = ajaxsyncBuffer::getCalls();
        if (count($calls)) {
            $callStr = "";
            foreach ($calls as $call) {
                $callStr .= ajaxsyncDataExchange::writeList($call) . "\n";
            }
            // Direct output, without using a view object - we are directly
            // communicating with the AjaxSync client here
            echo $callStr;
            if (!$loginStatus) {
                // This should never be reached, because checkLogin empties the queues -
                // but perhaps in future versions of AjaxSync messages to anonymous,
                // anonymous users will be allowed ...
                echo "OK: not logged in\n";
            }
        }
        else {
            // No calls yet - then wait!
            if ($errmsg = ajaxsyncCore::wait()) {
                echo $errmsg;
            }
            else {
                ajaxsyncSession::session_start(); // session could be accessed during wait (f.e. send), so re-open
                if ($loginMsg = ajaxsyncAccessControl::checkLogin()) {
                    echo $loginMsg;
                }
                // Now there should be at least one call, except if the signal is a
                // keepalive - but then simply nothing is done in the following loop
                $calls = ajaxsyncBuffer::getCalls();
                $callStr = "";
                foreach ($calls as $call) {
                    $callStr .= ajaxsyncDataExchange::writeList($call) . "\n";
                }
                // Direct output, without using a view object - we are directly
                // communicating with the AjaxSync client here
                echo $callStr;
                // close and update the session
                ajaxsyncSession::session_write_close();
            }
        }
    }
}
?>
