<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * This class provides the API for managing access control as well as for basic
 * authentication.
 *
 * It is a container for static functions.
 *
 * @author Michael Oberg
 */
final class ajaxsyncAccessControl {
    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __construct() { }

    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __clone() { }

    /**
     * Closes an AjaxSync session
     */
    public static function logout() {
        ajaxsyncBuffer::unregisterAll();
        ajaxsyncCore::removeAllListeners();
        unset(ajaxsyncSession::$values["AJAXSYNC_USER"],
                ajaxsyncSession::$values["AJAXSYNC_UID"],
                ajaxsyncSession::$values["AJAXSYNC_TIMEOUT"],
                ajaxsyncSession::$values["AJAXSYNC_LASTACCESS"]);
    }

    /**
     * Authenticates an user.
     *
     * This method requires an opened ajaxsyncSession object (i.e. call to
     * #ajaxsyncSession::session_start() before and #ajaxsyncSession::session_write_close()
     * after calling this method)
     * @param $user User
     * @param $password Password
     * @return True if login was successful, false otherwise
     */
    public static function login($user,$password) {
        $session = ajaxsyncSession::getSID(); // now existence of the session is guaranteed
	// start with a logout
        ajaxsyncAccessControl::logout();
        $useresc = mysql_escape_string($user);
        $password = mysql_escape_string($password);
        $u = ajaxsyncDB::selectSingle("select id, timeout from users where name = '$useresc' and pw = '$password'");
        if ($u) { // else access was denied
            ajaxsyncSession::$values["AJAXSYNC_USER"] = $user;
            ajaxsyncSession::$values["AJAXSYNC_UID"] = $u["id"];
            ajaxsyncSession::$values["AJAXSYNC_TIMEOUT"] = $u["timeout"];
            ajaxsyncSession::$values["AJAXSYNC_LASTACCESS"] = time();
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Checks session for a login timeout. Proceed with a complete logout if there was a
     * timout.
     *
     * This method requires an opened ajaxsyncSession object (i.e. call to
     * #ajaxsyncSession::session_start() before and #ajaxsyncSession::session_write_close()
     * after calling this method)
     */
    public static function checkLogin() {
        if (!ajaxsyncSession::$values["AJAXSYNC_USER"]) {
            return "OK: not logged in\n";
        }
        if (ajaxsyncSession::$values["AJAXSYNC_LASTACCESS"] + ajaxsyncSession::$values["AJAXSYNC_TIMEOUT"] < time()) {
            // timeout means logout
            ajaxsyncAccessControl::logout();
            // do not continue (except in init - exception is catched there)
            return "OK: not logged in\n";
        }
        else {
            ajaxsyncSession::$values["AJAXSYNC_LASTACCESS"] = time();
        }
    }

    /**
     * Creates a new AjaxSync user account. For now there are no restrictions in the choice
     * of username or password (f.e. character sets, even control characters are allowed),
     * except a maximum length of 50 ASCII characters.
     * @param $user Name of the user.
     * @param $password Password for login. Bear in mind that this is stored in cleartext in
     * the database.
     * @param $timeout Inactivity timeout.
     * @return True if the account could be created; false if the account already exists.
     * In case of a real database error an exception is thrown.
     */
    public static function useradd($user,$password,$timeout) {
        $user = mysql_escape_string($user);
        $password = mysql_escape_string($password);
        if (!is_numeric($timeout)) {
            throw new Exception("Failed: given timeout has to be a positive integer");
        }
        $timeout = (float)$timeout;
        if (((int)$timeout != $timeout) or $timeout < 0) {
            throw new Exception("Failed: given timeout is not a positive integer");
        }
        if (ajaxsyncDB::insertId("insert ignore into users (name,pw,timeout) values ('$user','$password',$timeout)")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Changes the password of a given user.
     * @param $user Name of the user.
     * @param $password Password for login.
     * @return True if the password could be changed; false if the account does not exist.
     * In case of a real database error an exception is thrown.
     */
    public static function passwd($user,$password) {
        $user = mysql_escape_string($user);
        $password = mysql_escape_string($password);
        if (ajaxsyncDB::changeAffected("update users set pw = '$password' where name = '$user'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Changes the inactivity timeout of the user.
     * @param $user Name of the user.
     * @param $timeout New inactivity timeout.
     * @return True if the timeout could be changed; false if the account does not exist.
     * In case of a real database error an exception is thrown.
     */
    public static function chtimeout($user,$timeout) {
        $user = mysql_escape_string($user);
        if (!is_numeric($timeout)) {
            throw new Exception("Failed: given timeout has to be a positive integer");
        }
        $timeout = (float)$timeout;
        if (((int)$timeout != $timeout) or $timeout < 0) {
            throw new Exception("Failed: given timeout is not a positive integer");
        }
        if (ajaxsyncDB::changeAffected("update users set timeout = '$timeout' where name = '$user'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Deletes an user account.
     *
     * Be aware that this operation may deny read access to the function for users
     * which have already registered handlers for it. But already existing registrations
     * will continue to work, and also sessions using this user account will continue to
     * work.
     * @param $user Name of the user.
     * @return True if the account could be deleted; false if the account does not exist.
     * In case of a real database error an exception is thrown.
     */
    public static function userdel($user) {
        $user = mysql_escape_string($user);
        ajaxsyncDB::change("update functions f, users u set f.owner = null where u.name = '$user' and u.id = f.owner");
        ajaxsyncDB::change("delete from groupmembers using groupmembers, users u where u.name = '$user' and u.id = groupmembers.user");
        if (ajaxsyncDB::changeAffected("delete from users where name = '$user'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Creates a new group. For now there are no restrictions in the choice
     * of group name (f.e. character sets, even control characters are allowed),
     * except a maximum length of 50 ASCII characters.
     * @param $group Name of the group.
     * @return True if the group could be created; false if the group already exists.
     * In case of a real database error an exception is thrown.
     */
    public static function groupadd($group) {
        if (!$group) return false;
        $group = mysql_escape_string($group);
        if (ajaxsyncDB::insertId("insert ignore into groups (name) values ('$group')")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Deletes a group.
     *
     * Be aware that this operation may deny read access to the function for users
     * which have already registered handlers for it. But already existing registrations
     * will continue to work!
     * @param $group Name of the group.
     * @return True if the group could be deleted; false if the group does not exist.
     * In case of a real database error an exception is thrown.
     */
    public static function groupdel($group) {
        $group = mysql_escape_string($group);
        ajaxsyncDB::change("update functions f, groups g set f.group = null where g.name = '$group' and g.id = f.group");
        ajaxsyncDB::change("delete from groupmembers using groupmembers, groups g where g.name = '$group' and g.id = groupmembers.group");
        if (ajaxsyncDB::changeAffected("delete from groups where name = '$group'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Adds a user to a group.
     * @param $group Name of the group.
     * @param $user Name of the user.
     * @return True if the user could be added to the group; false if the group or
     * the user does not exist. In case of a real database error an exception is
     * thrown.
     */
    public static function addmember($group,$user) {
        $group = mysql_escape_string($group);
        $user = mysql_escape_string($user);
        if (ajaxsyncDB::insertId("insert ignore into groupmembers (user,group) select u.id, g.id from users u, groups g where u.name = '$user' and g.name = '$group'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Removes a user from a group.
     *
     * Be aware that this operation may deny read access to the function for users
     * which have already registered handlers for it. But already existing registrations
     * will continue to work!
     * @param $group Name of the group.
     * @param $user Name of the user.
     * @return True if the user could be removed from the group; false if the group or
     * the user does not exist. In case of a real database error an exception is
     * thrown.
     */
    public static function delmember($group,$user) {
        $group = mysql_escape_string($group);
        $user = mysql_escape_string($user);
        if (ajaxsyncDB::changeAffected("delete from groupmembers using groupmembers m, users u, groups g where u.name = '$user' and g.name = '$group' and m.user = u.id and m.group = g.id")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Helper method. Checks if a given value is 0 or 1, throwing an exception
     * otherwise.
     * @param $i Given value.
     * @param $err Error message to be inserted in the exception.
     */
    private static function is_01($i,$err) {
        if ("$i" != "0" && "$i" != "1") {
            throw new Exception("Failed: $err");
        }
    }

    /**
     * Create a new function which can be called (write permission), and for which
     * you can register a handler (read permission).
     *
     * For now there are no restrictions in the choice of the functions name (f.e.
     * character sets, even control characters are allowed), except a maximum
     * length of 50 ASCII characters.
     * @param $fname Name of the function.
     * @param $owner Name of the owner.
     * @param $group Name of of the group.
     * @param $uread Read permission set for the owner.
     * @param $uwrite Write permission set for the owner.
     * @param $gread Read permission set for the group.
     * @param $gwrite Write permission set for the group.
     * @return True if the function could be created; false if the function does
     * already exist or the group or the user does not exist. In case of a real
     * database error an exception is thrown.
     */
    public static function addfunction($fname,$owner,$group,$uread,$uwrite,$gread,$gwrite) {
        $fname = mysql_escape_string($fname);
        $owner = mysql_escape_string($owner);
        $group = mysql_escape_string($group);
        self::is_01($uread,"uread is not 0 nor 1");
        self::is_01($uwrite,"uwrite is not 0 nor 1");
        self::is_01($gread,"gread is not 0 nor 1");
        self::is_01($gwrite,"gwrite is not 0 nor 1");
        if (ajaxsyncDB::insertId("insert ignore into functions (name,owner,`group`,uread,uwrite,gread,gwrite) select '$fname', u.id, g.id, $uread, $uwrite, $gread, $gwrite from users u, groups g where u.name = '$owner' and g.name = '$group'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Delete a function. Beforehand all queues which could hold calls of this function
     * are emptied and deleted.
     * @param $fname Name of the function.
     * @return True if the function could be deleted; false if the function does
     * not exist. In case of a real database error an exception is thrown.
     */
    public static function delfunction($fname) {
        $fname = mysql_escape_string($fname);
        ajaxsyncBuffer::deleteQueues($fname);
        if (ajaxsyncDB::changeAffected("delete from functions where name = '$fname'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Change the owner of a function.
     *
     * Be aware that this operation may deny read access to the function for users
     * which have already registered handlers for it. But already existing registrations
     * will continue to work!
     * @param $fname Name of the function.
     * @param $user Name of the new owner.
     * @return True if the owner could be changed; false if the function or the
     * new owner does not exist. In case of a real database error an exception is
     * thrown.
     */
    public static function chown($fname,$user) {
        $fname = mysql_escape_string($fname);
        $user = mysql_escape_string($user);
        if (ajaxsyncDB::changeAffected("update functions, users set owner = u.id where u.name = '$user' and f.name = '$fname'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Change the group of a function.
     *
     * Be aware that this operation may deny read access to the function for users
     * which have already registered handlers for it. But already existing registrations
     * will continue to work!
     * @param $fname Name of the function.
     * @param $group Name of the new group.
     * @return True if the group could be changed; false if the function or the
     * new group does not exist. In case of a real database error an exception is
     * thrown.
     */
    public static function chgrp($fname,$group) {
        $fname = mysql_escape_string($fname);
        $group = mysql_escape_string($group);
        if (ajaxsyncDB::changeAffected("update functions, groups set group = g.id where g.name = '$group' and f.name = '$fname'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Change the permissions of a function.
     *
     * Be aware that this operation may deny read access to the function for users
     * which have already registered handlers for it. But already existing registrations
     * will continue to work!
     * @param $fname Name of the function.
     * @param $uread Read permission set for the owner.
     * @param $uwrite Write permission set for the owner.
     * @param $gread Read permission set for the group.
     * @param $gwrite Write permission set for the group.
     * @return True if the permissions could be changed; false if the function does
     * not exist. In case of a real database error an exception is thrown.
     */
    public static function chmod($fname,$uread,$uwrite,$gread,$gwrite) {
        $fname = mysql_escape_string($fname);
        self::is_01($uread,"uread is not 0 nor 1");
        self::is_01($uwrite,"uwrite is not 0 nor 1");
        self::is_01($gread,"gread is not 0 nor 1");
        self::is_01($gwrite,"gwrite is not 0 nor 1");
        if (ajaxsyncDB::changeAffected("update functions set uread=$uread,uwrite=$uwrite,gread=$gread,gwrite=$gwrite) where name = '$fname'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Search group. Added for jQuery Autocomplete.
     *
     * @param $group Substring of the name of the group.
     * @return List of matching group names.
     */
    public static function searchgroup($group) {
        $group = mysql_escape_string($group);
        $len = strlen($group);
        return ajaxsyncDB::selectMultiple("select name from groups where substring(name,1,$len) = binary '$group'");
    }

    /**
     * Search user. Added for jQuery Autocomplete.
     *
     * @param $user Substring of the name of the user.
     * @return List of matching user names.
     */
    public static function searchuser($user) {
        $user = mysql_escape_string($user);
        $len = strlen($user);
        return ajaxsyncDB::selectMultiple("select name from users where substring(name,1,$len) = binary '$user' and status = 2");
    }

    /**
     * Search function. Added for jQuery Autocomplete.
     *
     * @param $user Substring of the name of the function.
     * @return List of matching function names.
     */
    public static function searchfunction($function) {
        $function = mysql_escape_string($function);
        $len = strlen($function);
        return ajaxsyncDB::selectMultiple("select name from functions where substring(name,1,$len) = binary '$function'");
    }

    /**
     * List users and mark group members. Added for jQuery Multiselect.
     *
     * @param $group Name of the group.
     * @return List of all users with marks.
     */
    public static function usersGroupMembers($group) {
        $group = mysql_escape_string($group);
        return ajaxsyncDB::selectMultiple("select u.name, isnull(m.group) = 0 selected from groups g inner join users u on g.name = '$group' left outer join groupmembers m on u.id = m.user and m.group = g.id");
    }

    /**
     * List groups and mark membership. Added for jQuery Multiselect.
     *
     * @param $user Name of the user.
     * @return List of all groups with marks.
     */
    public static function groupUserMembers($user) {
        $user = mysql_escape_string($user);
        return ajaxsyncDB::selectMultiple("select g.name, isnull(m.user) = 0 selected from users u inner join groups g on u.name = '$user' left outer join groupmembers m on g.id = m.group and m.user = u.id");
    }

    /**
     * Get user info (for now, only timeout).
     *
     * @param $user Name of the user.
     * @return List of user properties (for now, only timeout).
     */
    public static function userInfo($user) {
        $user = mysql_escape_string($user);
        return ajaxsyncDB::selectSingle("select timeout from users where name = '$user'");
    }

    /**
     * Get function info.
     *
     * @param $fname Name of the function.
     * @return List of function properties.
     */
    public static function functionInfo($fname) {
        $fname = mysql_escape_string($fname);
        return ajaxsyncDB::selectSingle("select u.name owner, g.name `group`, f.uread, f.uwrite, f.gread, f.gwrite from functions f left outer join users u on u.id = f.owner left outer join groups g on g.id = f.group where f.name = '$fname'");
    }

    /**
     * Change membership, then list users and mark group members.
     * Added for jQuery Multiselect.
     *
     * @param $group Name of the group.
     * @param $input List of group members.
     * @return List of user names with membership marks.
     */
    public static function changeGroupMembers($group,$members) {
        $group = mysql_escape_string($group);
        ajaxsyncDB::change("delete from groupmembers using groupmembers, groups g where g.name = '$group' and g.id = groupmembers.group");
        for ($i = 0; $i < count($members); $i++) {
            $user = mysql_escape_string($members[$i]);
            ajaxsyncDB::change("insert into groupmembers (`group`, user) select g.id, u.id from groups g, users u where g.name = '$group' and u.name = '$user'");
        }
        return ajaxsyncDB::selectMultiple("select u.name, isnull(m.group) = 0 selected from groups g inner join users u on g.name = '$group' left outer join groupmembers m on u.id = m.user and m.group = g.id");
    }

    /**
     * Change membership, then list groups and mark membership.
     * Added for jQuery Multiselect.
     *
     * @param $user Name of the group.
     * @param $timeout Server timeout.
     * @param $passwd Optional new password.
     * @param $input List of groups the user is member of.
     * @return List of group names with membership marks.
     */
    public static function changeUserMembership($user,$timeout,$passwd,$memberof) {
        $user = mysql_escape_string($user);
        $timeout = mysql_escape_string($timeout);
        $passwd = mysql_escape_string($passwd);
        if ($passwd)
            ajaxsyncDB::change("update users set timeout = $timeout, pw = '$passwd' where name = '$user'");
        else
            ajaxsyncDB::change("update users set timeout = $timeout where name = '$user'");
        ajaxsyncDB::change("delete from groupmembers using groupmembers, users u where u.name = '$user' and u.id = groupmembers.user");
        for ($i = 0; $i < count($memberof); $i++) {
            $group = mysql_escape_string($memberof[$i]);
            ajaxsyncDB::change("insert into groupmembers (`group`, user) select g.id, u.id from groups g, users u where g.name = '$group' and u.name = '$user'");
        }
        return ajaxsyncDB::selectMultiple("select g.name, isnull(m.user) = 0 selected from users u inner join groups g on u.name = '$user' left outer join groupmembers m on g.id = m.group and m.user = u.id");
    }

    /**
     * Change function (owner, group, permissions).
     *
     * @param $fname Name of the function.
     * @param $owner Name of the owner (user).
     * @param $group Name of the group.
     * @param $uread Owners read permission.
     * @param $uwrite Owners write permission.
     * @param $gread Group read permission.
     * @param $gwrite Group write permission.
     * @return True if the owner/group/permissions could be changed; false if
     * the function/owner/group does not exist. In case of a real database
     * error an exception is thrown.
     */
    public static function changeFunction($fname,$owner,$group,$uread,$uwrite,$gread,$gwrite) {
        $fname = mysql_escape_string($fname);
        $owner = mysql_escape_string($owner);
        $group = mysql_escape_string($group);
        self::is_01($uread,"uread is not 0 nor 1");
        self::is_01($uwrite,"uwrite is not 0 nor 1");
        self::is_01($gread,"gread is not 0 nor 1");
        self::is_01($gwrite,"gwrite is not 0 nor 1");
        if (ajaxsyncDB::changeAffected("update functions f, users u, groups g set f.owner = u.id, f.group = g.id, f.uread=$uread, f.uwrite=$uwrite, f.gread=$gread, f.gwrite=$gwrite where f.name = '$fname' and u.name = '$owner' and g.name = '$group'")) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * List all functions readable by the given user.
     *
     * @param $user Name of the user.
     * @return List of functions with ownership mark.
     */
    public static function functionList($user) {
        $user = mysql_escape_string($user);
        return ajaxsyncDB::selectMultiple("select distinct f.name fname, f.owner = u.id isowned from functions f, users u, groupmembers m where u.name = '$user' and (f.owner = u.id or (f.group = m.group and m.user = u.id))");
    }

    /**
     * List all users which can potentially interact with the given user. That means, every
     * member from groups in which the given user is member, and each member of a group of
     * the functions owned by the given user, and each function owner whose function is accessible
     * by a group which has the given user as a member. (Exactly, this does not respect the possibility
     * that a group is not associated with a function. Alas, this possible misconfiguration is
     * ignored for performance reasons.)
     *
     * @param $user Name of the user.
     * @return List of user names with online status. Being online means nothing except that
     * this user has made an authorized wait call less than twice AJAXSYNC_KEEPALIVE_DELAY
     * periods ago. It says nothing about the number of open sessions or their usage.
     */
    public static function friends($user) {
        $user = mysql_escape_string($user);
	$u = ajaxsyncDB::selectSingle("select id from users where name = '$user'");
	$user = $u["id"];
        return ajaxsyncDB::selectMultiple("select distinct u.name, (u.lastaccess >= date_sub(now(), interval " . (2 * AJAXSYNC_KEEPALIVE_DELAY) . " second)) online from functions f, groupmembers g, users u where f.owner = $user and g.group = f.group and g.user = u.id and u.id <> $user union select distinct u.name, (u.lastaccess >= date_sub(now(), interval " . (2 * AJAXSYNC_KEEPALIVE_DELAY) . " second)) online from groupmembers g1, groupmembers g2, users u where g1.user = $user and g1.group = g2.group and g2.user = u.id and u.id <> $user union select distinct u2.name, (u2.lastaccess >= date_sub(now(), interval " . (2 * AJAXSYNC_KEEPALIVE_DELAY) . " second)) online from functions f, groupmembers g, users u, users u2 where f.owner = u2.id and g.group = f.group and g.user = u.id and u.id = $user and f.owner <> u.id");
    }
}
?>
