<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * With this class the Synchronizer can be started and stopped.
 * It is a container for static functions.
 *
 * WARNING: this code will only run on POSIX systems.
 *
 * @author Michael Oberg
 */
final class ajaxsyncCtl {
    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __construct() { }

    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __clone() { }

    /**
     * Starts the Synchronizer service and stores the PID in the database.
     */
    public static function startSynchronizer() {
        $p = proc_open(AJAXSYNC_SYNCHRONIZER_CMD,
                array(0 => array("pipe","r"), 1 => array("pipe","w")),
                $pipes);
        if (is_resource($p)) {
            $in = $pipes[0];
            $out = $pipes[1];
            fwrite($in, AJAXSYNC_PORT . "\n");
            fwrite($in, AJAXSYNC_KEEPALIVE_DELAY . "\n");
            fwrite($in, AJAXSYNC_LOGLEVEL . "\n");
            fwrite($in, AJAXSYNC_PASSWORD . "\n");
            fwrite($in, AJAXSYNC_LOGFILE . "\n");
            $answer = fread($out, 1024);
            fclose($in);
            fclose($out);
            proc_close($p);
	    if (preg_match("/port=([0-9]+),delay=([0-9]+),loglevel=([0-9]+),logfile=([^,]*).*PID: ([0-9]+)/",$answer,$matches)) {
                $port = $matches[1];
                $delay = $matches[2];
                $loglevel = $matches[3];
                $logfile = mysql_escape_string($matches[4]);
                $pid = $matches[5];
                ajaxsyncDB::change("insert into synchronizer (port, delay, loglevel, logfile, pid, started) values ($port, $delay, $loglevel, '$logfile', $pid, now())");
	    }
            else
                throw new Exception("Failed: Could not start Synchronizer (output: $answer)");
        }
        else {
            throw new Exception("Failed: Could not start Synchronizer");
        }
    }

    /**
     * Stops the Synchronizer service.
     */
    public static function stopSynchronizer() {
        if ($ret = ajaxsyncDB::selectSingle("select pid from sessioncontainer order by started desc limit 1")) {
            $pid = $ret["pid"];
            if (!posix_kill($pid, 9)) {
                throw new Exception("Failed: Could not stop Synchronizer");
            }
        }
    }

    /**
     * Checks the Synchronizer service.
     */
    public static function checkSynchronizer() {
        if ($ret = ajaxsyncDB::selectSingle("select pid from sessioncontainer order by started desc limit 1")) {
            $pid = $ret["pid"];
            return posix_kill($pid, 0);
        }
    }
}
?>
