<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * This is a simple Interface to mySQL.
 * It is a container for static functions.
 *
 * @author Michael Oberg
 */
final class ajaxsyncDB {
    /**
     * mySQL connection status. The first statement executed with a method of this
     * class enforces a connection attempt.
     */
    private static $connected = false;

    /**
     * Connection to mySQL.
     */
    private static $mysql;

    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __construct() { }

    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __clone() { }

    /**
     * Connect to the mySQL database mentioned in ajaxsyncConfig.php. Throw an exeption if
     * the connection attempt fails. Implicitely called from every statement made with this
     * class.
     * @return Nothing.
     */
    private static function connect() {
        if (self::$connected) {
            return; // If already connected, do nothing
        }
        if (!(self::$mysql = mysql_connect(AJAXSYNC_DBSERVER, AJAXSYNC_DBUSER, AJAXSYNC_DBPW))) {
            throw new Exception('Failed: Could not connect to the server');
        }
        if (!mysql_select_db(AJAXSYNC_DBNAME, self::$mysql)) {
            throw new Exception('Failed: Could not connect to the database');
        }
        self::$connected = true;
    }

    /**
     * Execute a given select statement with more than one result row expected. Throw
     * an exception if the execution fails.
     * @param $sql The select statement in a string.
     * @return A list of rows. Each row is stored in an associative array.
     */
    public static function selectMultiple($sql) {
        self::connect();
        if (!($result = mysql_query($sql, self::$mysql))) {
            throw new Exception("Failed: " . mysql_error() . " - The query was: " . $sql);
        }
        $res = array();
        while ($row = mysql_fetch_assoc($result)) {
            array_push($res, $row);
        }
        return $res;
    }

    /**
     * Execute a given select statement with exactly one result row expected. Throw
     * an exception if the execution fails.
     * @param $sql The select statement in a string.
     * @return A single row stored as an associative array.
     */
    public static function selectSingle($sql) {
        self::connect();
        if (!($result = mysql_query($sql, self::$mysql))) {
            throw new Exception("Failed: " . mysql_error() . " - The query was: " . $sql);
        }
        if ($row = mysql_fetch_assoc($result)) {
            return $row;
        }
        return false;
    }

    /**
     * Execute an update / insert / delete statement. Throw an exception if the
     * execution fails.
     * @param $sql SQL statement
     * @return Nothing
     */
    public static function change($sql) {
        self::connect();
        if (!($result = mysql_query($sql, self::$mysql))) {
            throw new Exception("Failed: " . mysql_error() . " - The query was: " . $sql);
        }
    }
    
    /**
     * Execute an update / insert / delete statement. Throw an exception if the
     * execution fails.
     * @param $sql SQL statement
     * @return Count of affected rows
     */
    public static function changeAffected($sql) {
        self::connect();
        if (!($result = mysql_query($sql, self::$mysql))) {
            throw new Exception("Failed: " . mysql_error() . " - The query was: " . $sql);
        }
        return mysql_affected_rows(self::$mysql);
    }

    /**
     * Execute an insert statement. Throw an exception if the execution fails.
     * @param $sql SQL statement
     * @return ID of the last inserted row
     */
    public static function insertId($sql) {
        self::connect();
        if (!($result = mysql_query($sql, self::$mysql))) {
            throw new Exception("Failed: " . mysql_error() . " - The query was: " . $sql);
        }
        return mysql_insert_id(self::$mysql);
    }

    /**
     * Alias for #change - execute an insert statement. Throw an exception if the execution fails.
     * @param $sql SQL statement
     */
    public static function insert($sql) {
        self::change($sql);
    }
}
?>
