<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 * 
 * This file is part of the AjaxSync framework.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * This class manages the optional logging (debugging exactly) of the AjaxSync
 * controllers. It is used for dumping the entire HTTP traffic into a file
 * (which is <b>extremely</b> resource extensive). The hooks for thin class are
 * in the index.php file; configuration is done in the #config/ajaxsyncConfig.php
 * file.
 *
 * @author Michael Oberg
 */
final class ajaxsyncLogger {
    /**
     * Logging is done before and after calling the controller. In order to put
     * the two log entries together, a unique id of the call is created and stored
     * here.
     */
    private static $id;

    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __construct() { }

    /**
     * A private constructor/clone does make creating instances of this class impossible.
     */
    private function __clone() { }

    /**
     * Log incoming traffic to the server: POST and GET parameters, and the AjaxSync
     * session id.
     */
    public function startLog() {
        ob_start();
        self::$id = md5(uniqid(mt_rand(), true));
        $date = new DateTime();
        file_put_contents(AJAXSYNC_PHPLOGFILE,
                "--- START: " . $date->format("Y-m-d H:i:s") . "\n"
                . "--- Call ID: " . self::$id . "\n"
                . "--- AJAXSYNC_SESSION: " . $_COOKIE["AJAXSYNC_SESSION"] . "\n"
                . "--- GET: " . join("&", $_GET) . "\n"
                . "--- POST: " . join("&", $_POST) . "\n---------\n",
                FILE_APPEND);
    }

    /**
     * Write an explicite log message in the AjaxSync log file.
     */
    public function writeMessage($message) {
        $date = new DateTime();
        file_put_contents(AJAXSYNC_PHPLOGFILE,
                "--- ERROR: " . $date->format("Y-m-d H:i:s") . "\n"
                . "--- Call ID: " . self::$id . "\n"
                . "--- Message: " . $message . "\n---------\n",
                FILE_APPEND);
    }

    /**
     * Log outgoing traffic from the server.
     */
    public function stopLog() {
        $date = new DateTime();
        file_put_contents(AJAXSYNC_PHPLOGFILE,
                "--- STOP: " . $date->format("Y-m-d H:i:s") . "\n"
                . "--- Call ID: " . self::$id . "\n"
                . "--- Answer: " . ob_get_contents() . "\n---------\n",
                FILE_APPEND);
        ob_end_flush();
    }
}
?>
