<?php
/****************************************************************************
 * Copyright (C) 2010  Michael Oberg
 *
 * This file is part of the AjaxSync framework.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ****************************************************************************/

/**
 * @file index.php
 * The index.php file is the entry point of an AjaxSync application. Every access
 * to the application is redirected by the rules in the .htaccess file to the
 * index.php, providing a parameter #$url which holds the original, relative URL
 * below the directory containing the index.php. This URL is changed in a
 * controller name by replacing slashes with underscores, then the controller is
 * loaded using the autoloader and executed.
 *
 * @author Michael Oberg
 */

ini_set('display_errors','Off'); // Warning, display_errors would corrupt Ajax answers

require_once 'ajaxsyncAppdir.php';

require_once AJAXSYNC_APPDIR . 'config/ajaxsyncConfig.php';
require_once AJAXSYNC_APPDIR . 'config/ajaxsyncAppConfig.php';
require_once AJAXSYNC_APPDIR . 'functions/autoload.php';

$url = $_GET["url"];
$url = preg_replace(array("/^\\//","/index.php$/","/\\/$/"), "", $url);
$url = str_replace("/", "_", $url);

// get controller
if ($url) {
    if (file_exists(AJAXSYNC_APPDIR . "classes/controller/$url.php")) {
        $controller = new $url;
    }
    else {
        require_once AJAXSYNC_APPDIR . 'template/error404.php';
        exit(0);
    }
}
else {
    $controller = new start;
}
if (AJAXSYNC_PHPLOG)
    ajaxsyncLogger::startLog();
try {
    // because in PHP 5.3 disabling the magic quotes is deprecated,
    // we have to strip the quotes manually!
    if (function_exists(get_magic_quotes_gpc) && get_magic_quotes_gpc())
        $_POST = array_map("stripslashes",$_POST);
    $controller->run($_POST);
    if (AJAXSYNC_PHPLOG)
        ajaxsyncLogger::stopLog();
}
catch (Exception $e) {
    // Every uncatched or re-throwed error encountered in an AjaxSync controller is displayed
    // and re-throwed so that it is visible in the Apache logs, too.
    echo $e->getMessage() . "\n";
    if (AJAXSYNC_PHPLOG)
        ajaxsyncLogger::stopLog();
    throw $e;
}
?>
