/*
 * Copyright (C) 2010  Michael Oberg
 *
 * This file is part of the AjaxSync framework.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * @fileoverview This is the AjaxSync Administrator jQuery interface.
 *
 * @author Michael Oberg
 */

$(function(){
    $.ajaxSetup({
        beforeSend: function() {
            $('#Spinner').show();
        },
        complete: function() {
            $('#Spinner').hide();
        },
        error: function(x,e,t) {
            if (e == 'error')
                $('#Error').html('HTTP status: '+x.status);
            else if (e == 'parsererror')
                $('#Error').html(t);
            else
                $('#Error').html(e);
            $('#Error').dialog('open');
        }
    });
    $('#Error').dialog({
        autoOpen: false,
        width: 400,
        buttons: {
            "OK": function() {
                $(this).html("");
                $(this).dialog("close");
            }
        }
    });
    $(".multiselect").multiselect();
    createMultiple('members');
    createMultiple('memberof');
    $('.link-button, ul#icons li').hover(
        function() {
            $(this).addClass('ui-state-hover');
        },
        function() {
            $(this).removeClass('ui-state-hover');
        }
        );
    $('#tabs').tabs();
    $('#creategroup').dialog({
        autoOpen: false,
        width: 320,
        buttons: {
            "Ok": function() {
                var groupname = $('#newgroupname').val();
                var dialogPtr = $(this);
                $.ajax({
                    type: 'POST',
                    url: 'creategroup',
                    data: 'group='+groupname,
                    dataType: 'json',
                    success: function(data) {
                        $('#newgroupname').val("");
                        dialogPtr.dialog("close");
                    }
                });
            },
            "Cancel": function() {
                $(this).dialog("close");
            }
        }
    });
    $('#creategroup_link').click(function(){
        $('#creategroup').dialog('open');
        return false;
    });
    var groupnameSelected = false;
    $('#delgroup_link').click(function(){
        if (!groupnameSelected) {
            $('#Error').html('No group selected.');
            $('#Error').dialog('open');
            return false;
        }
        $.ajax({
            type: 'POST',
            url: 'delgroup',
            data: 'group='+$('#groupname').val(),
            dataType: 'json',
            success: function(data) {
                $('#groupname').val("");
                groupnameSelected = false;
                createMultiple('members');
            }
        });
        return false;
    });
    $('#cancelgroup_link').click(function(){
        if (!groupnameSelected) {
            $('#Error').html('No group selected.');
            $('#Error').dialog('open');
            return false;
        }
        $.ajax({
            type: 'POST',
            url: 'groupmembers',
            data: 'group='+$('#groupname').val(),
            dataType: 'json',
            success: function(data) {
                createMultiple('members',data);
            }
        });
        return false;
    });
    $('#savegroup_link').click(function(){
        if (!groupnameSelected) {
            $('#Error').html('No group selected.');
            $('#Error').dialog('open');
            return false;
        }
        var param = 'group='+$('#groupname').val()+'&members=';
        var id = 'members';
        var url = 'changemembers';
        sendMultiple(id,url,param);
        return false;
    });
    $("#groupname").autocomplete({
        source:"searchgroup",
        minLength: 0,
        open: function(e,ui) {
            groupnameSelected = false;
            createMultiple('members');
        },
        select: function(e,ui) {
            groupnameSelected = true;
            $.ajax({
                type: 'POST',
                url: 'groupmembers',
                data: 'group='+ui.item.value,
                dataType: 'json',
                success: function(data) {
                    createMultiple('members',data);
                }
            });
        },
        close: function(e,ui) {
            if(!groupnameSelected)
                createMultiple('members');
        }
    });
    $('#createuser').dialog({
        autoOpen: false,
        width: 350,
        buttons: {
            "Ok": function() {
                var username = $('#newusername').val();
                var passwd = $('#newuserpasswd').val();
                var timeout = $('#newtimeout').val();
                var dialogPtr = $(this);
                $.ajax({
                    type: 'POST',
                    url: 'createuser',
                    data: 'user='+escape(username)
                    +'&passwd='+escape(passwd)
                    +'&timeout='+escape(timeout),
                    dataType: 'json',
                    success: function(data) {
                        $('#newusername').val("");
                        $('#newuserpasswd').val("");
                        $('#newtimeout').val("60");
                        dialogPtr.dialog("close");
                    }
                });
                $(this).dialog("close");
            },
            "Cancel": function() {
                $('#newusername').val("");
                $('#newuserpasswd').val("");
                $('#newtimeout').val("60");
                $(this).dialog("close");
            }
        }
    });
    $('#createuser_link').click(function(){
        $('#createuser').dialog('open');
        return false;
    });
    var usernameSelected = false;
    $('#deluser_link').click(function(){
        if (!usernameSelected) {
            $('#Error').html('No user selected.');
            $('#Error').dialog('open');
            return false;
        }
        $.ajax({
            type: 'POST',
            url: 'deluser',
            data: 'user='+$('#username').val(),
            dataType: 'json',
            success: function(data) {
                $('#username').val("");
                $('#usertimeout').val("");
                $('#userpw').val("");
                usernameSelected = false;
                createMultiple('memberof');
            }
        });
        return false;
    });
    $('#canceluser_link').click(function(){
        if (!usernameSelected) {
            $('#Error').html('No user selected.');
            $('#Error').dialog('open');
            return false;
        }
        $.ajax({
            type: 'POST',
            url: 'userinfo',
            data: 'user='+$('#username').val(),
            dataType: 'json',
            success: function(data) {
                $('#usertimeout').val(data.timeout);
                $('#userpw').val("");
                createMultiple('memberof',data.list);
            }
        });
        return false;
    });
    $('#saveuser_link').click(function(){
        if (!usernameSelected) {
            $('#Error').html('No user selected.');
            $('#Error').dialog('open');
            return false;
        }
        var param = 'user='+escape($('#username').val())
        + '&passwd='+escape($('#userpw').val())
        + '&timeout='+escape($('#usertimeout').val())
        + '&memberof=';
        var id = 'memberof';
        var url = 'changemembership';
        sendMultiple(id,url,param);
        return false;
    });
    $("#username").autocomplete({
        source:"searchuser",
        minLength: 0,
        open: function(e,ui) {
            usernameSelected = false;
            $('#usertimeout').val("");
            $('#userpw').val("");
            createMultiple('memberof');
        },
        select: function(e,ui) {
            usernameSelected = true;
            $.ajax({
                type: 'POST',
                url: 'userinfo',
                data: 'user='+ui.item.value,
                dataType: 'json',
                success: function(data) {
                    $('#usertimeout').val(data.timeout);
                    $('#userpw').val("");
                    createMultiple('memberof',data.list);
                }
            });
        },
        close: function(e,ui) {
            if(!usernameSelected) {
                $('#usertimeout').val("");
                $('#userpw').val("");
                createMultiple('memberof');
            }
        }
    });
    var usernameFuncSelected = false;
    $("#fowner").autocomplete({
        source:"searchuser",
        minLength: 0,
        open: function(e,ui) {
            usernameFuncSelected = false;
        },
        select: function(e,ui) {
            usernameFuncSelected = true;
        }
    });
    var groupnameFuncSelected = false;
    $("#fgroup").autocomplete({
        source:"searchgroup",
        minLength: 0,
        open: function(e,ui) {
            groupnameFuncSelected = false;
        },
        select: function(e,ui) {
            groupnameFuncSelected = true;
        }
    });
    var functionnameSelected = false;
    $("#fname").autocomplete({
        source:"searchfunction",
        minLength: 0,
        open: function(e,ui) {
            functionnameSelected = false;
        },
        select: function(e,ui) {
            functionnameSelected = true;
            $.ajax({
                type: 'POST',
                url: 'functioninfo',
                data: 'function='+ui.item.value,
                dataType: 'json',
                success: function(data) {
                    $('#fowner').val(data.owner);
                    $('#fgroup').val(data.group);
                    usernameFuncSelected = true;
                    groupnameFuncSelected = true;
                    $('#uread').attr('checked',data.uread=='1');
                    $('#uwrite').attr('checked',data.uwrite=='1');
                    $('#gread').attr('checked',data.gread=='1');
                    $('#gwrite').attr('checked',data.gwrite=='1');
                }
            });
        },
        close: function(e,ui) {
            if(!functionnameSelected) {
                $('#fowner').val("");
                $('#fgroup').val("");
                usernameFuncSelected = false;
                groupnameFuncSelected = false;
                $('#uread').attr('checked',false);
                $('#uwrite').attr('checked',false);
                $('#gread').attr('checked',false);
                $('#gwrite').attr('checked',false);
            }
        }
    });
    $('#delfunction_link').click(function(){
        if (!functionnameSelected) {
            $('#Error').html('No function selected.');
            $('#Error').dialog('open');
            return false;
        }
        $.ajax({
            type: 'POST',
            url: 'delfunction',
            data: 'function='+$('#fname').val(),
            dataType: 'json',
            success: function(data) {
                $('#fname').val("");
                $('#fowner').val("");
                $('#fgroup').val("");
                usernameFuncSelected = false;
                groupnameFuncSelected = false;
                $('#uread').attr('checked',false);
                $('#uwrite').attr('checked',false);
                $('#gread').attr('checked',false);
                $('#gwrite').attr('checked',false);
            }
        });
        return false;
    });
    $('#cancelfunction_link').click(function(){
        if (!functionnameSelected) {
            $('#Error').html('No function selected.');
            $('#Error').dialog('open');
            return false;
        }
        $.ajax({
            type: 'POST',
            url: 'functioninfo',
            data: 'function='+$('#fname').val(),
            dataType: 'json',
            success: function(data) {
                $('#fowner').val(data.owner);
                $('#fgroup').val(data.group);
                usernameFuncSelected = true;
                groupnameFuncSelected = true;
                $('#uread').attr('checked',data.uread=='1');
                $('#uwrite').attr('checked',data.uwrite=='1');
                $('#gread').attr('checked',data.gread=='1');
                $('#gwrite').attr('checked',data.gwrite=='1');
            }
        });
        return false;
    });
    $('#savefunction_link').click(function(){
        if (!functionnameSelected) {
            $('#Error').html('No function selected.');
            $('#Error').dialog('open');
            return false;
        }
        if (!(usernameFuncSelected && groupnameFuncSelected)) {
            $('#Error').html('You have to select both user and group.');
            $('#Error').dialog('open');
            return false;
        }
        var param = 'function='+escape($('#fname').val())
        + '&owner='+escape($('#fowner').val())
        + '&group='+escape($('#fgroup').val())
        + '&uread='+($('#uread').attr('checked') ? '1' : '0')
        + '&uwrite='+($('#uwrite').attr('checked') ? '1' : '0')
        + '&gread='+($('#gread').attr('checked') ? '1' : '0')
        + '&gwrite='+($('#gwrite').attr('checked') ? '1' : '0');
        $.ajax({
            type: 'POST',
            url: 'changefunction',
            data: param,
            dataType: 'json',
            success: function(data) {
                $('#fowner').val(data.owner);
                $('#fgroup').val(data.group);
                usernameFuncSelected = true;
                groupnameFuncSelected = true;
                $('#uread').attr('checked',data.uread=='1');
                $('#uwrite').attr('checked',data.uwrite=='1');
                $('#gread').attr('checked',data.gread=='1');
                $('#gwrite').attr('checked',data.gwrite=='1');
            }
        });
        return false;
    });
    var newUsernameFuncSelected = false;
    $("#newfowner").autocomplete({
        source:"searchuser",
        minLength: 0,
        open: function(e,ui) {
            newUsernameFuncSelected = false;
        },
        select: function(e,ui) {
            newUsernameFuncSelected = true;
        }
    });
    var newGroupnameFuncSelected = false;
    $("#newfgroup").autocomplete({
        source:"searchgroup",
        minLength: 0,
        open: function(e,ui) {
            newGroupnameFuncSelected = false;
        },
        select: function(e,ui) {
            newGroupnameFuncSelected = true;
        }
    });
    $('#createfunction').dialog({
        autoOpen: false,
        width: 470,
        buttons: {
            "Ok": function() {
                if (!(newUsernameFuncSelected && newGroupnameFuncSelected)) {
                    $('#Error').html('You have to select both user and group.');
                    $('#Error').dialog('open');
                    return;
                }
                var param = 'function='+escape($('#newfname').val())
                + '&owner='+escape($('#newfowner').val())
                + '&group='+escape($('#newfgroup').val())
                + '&uread='+($('#newuread').attr('checked') ? '1' : '0')
                + '&uwrite='+($('#newuwrite').attr('checked') ? '1' : '0')
                + '&gread='+($('#newgread').attr('checked') ? '1' : '0')
                + '&gwrite='+($('#newgwrite').attr('checked') ? '1' : '0');
                $.ajax({
                    type: 'POST',
                    url: 'createfunction',
                    data: param,
                    dataType: 'json',
                    success: function(data) {
                        $('#newfname').val("");
                        $('#newfowner').val("");
                        $('#newfgroup').val("");
                        newUsernameFuncSelected = false;
                        newGroupnameFuncSelected = false;
                        $('#newuread').attr('checked',false);
                        $('#newuwrite').attr('checked',false);
                        $('#newgread').attr('checked',false);
                        $('#newgwrite').attr('checked',false);
                    }
                });
                $(this).dialog("close");
            },
            "Cancel": function() {
                $('#newfname').val("");
                $('#newfowner').val("");
                $('#newfgroup').val("");
                newUsernameFuncSelected = false;
                newGroupnameFuncSelected = false;
                $('#newuread').attr('checked',false);
                $('#newuwrite').attr('checked',false);
                $('#newgread').attr('checked',false);
                $('#newgwrite').attr('checked',false);
                $(this).dialog("close");
            }
        }
    });
    $('#createfunction_link').click(function(){
        $('#createfunction').dialog('open');
        return false;
    });
});

function createMultiple(id,data) {
    var str = '<select id="' + id + '" class="multiselect" multiple="multiple" style="display: none;">';
    if (data && data.length)
        for (var i = 0; i < data.length; i++) {
            if (data[i].selected == "1")
                str += '<option value="' + data[i].name + '" selected="selected">' + data[i].name + '</option>';
            else
                str += '<option value="' + data[i].name + '">' + data[i].name + '</option>';
        }
    str += '</select>';
    $('#'+id).multiselect('destroy');
    $('#'+id).replaceWith(str);
    $('#'+id).multiselect();
}

function sendMultiple(id,url,param) {
    var list = [];
    $('#'+id+' :selected').each(function(i,s){
        list[i] = $(s).text();
    });
    var str = JSON.stringify(list);
    $.ajax({
        type: 'POST',
        url: url,
        data: param+str,
        dataType: 'json',
        success: function(data) {
            createMultiple(id,data);
        }
    });
}