/**
 * @class This class creates a timestamp object. The timestamp is returned
 * in german format.
 * @constructor
 */
function chatTimestamp() {
    /**
     * @function Private function for adding leading zeros to a number.
     */
    function zp(num,cnt) { // zero padding
        var numstr = num + '';
        while (numstr.length < cnt)
            numstr = '0' + numstr;
        return numstr;
    }
    this.now = function() {
        var d = new Date();
        var dstr = zp(d.getDate(),2)
            + '.'
            + zp(d.getMonth() + 1,2)
            + '.'
            + d.getFullYear()
            + ' '
            + zp(d.getHours(),2)
            + ':'
            + zp(d.getMinutes(),2)
            + ':'
            + zp(d.getSeconds(),2);
        return dstr;
    }
}

/**
 * @class This class creates a chatroom object. That object creates
 * message div, editor div including textarea, send button associated
 * with the AjaxSync send call, and AjaxSync handler function with
 * association to the message div.
 * @param j Index key of the chatroom in array app.functionList.
 * @param n Name of the chatroom.
 * @param o "Owned", that means the handler function is owned by the
 * current user, therefore the chatroom can be deleted by the current
 * user.
 * @constructor
 */
function chatroomObject(j,n,o) {
    var chatroom = n;
    var owned = o;
    var hidden = false;
    var blinker = false;
    var blinkOn = false;
    var newmessage = false;
    var ts = new chatTimestamp();
    document.getElementById("editarea").innerHTML
        += '<div id="tab_' + chatroom + '" class="chatTab">\n'
        + '    <div id="msg_' + chatroom + '" class="chatOutput">\n'
        + '    </div>\n'
        + '    <div id="edit_' + chatroom + '">\n'
        + '        <textarea class="tinymce" style="width: 100%" id="send_' + chatroom + '"></textarea>\n'
        + '        <table cellspacing="0" style="width: 100%; margin: 4px 0px 4px 0px; padding: 0px;">\n'
        + '            <tr>\n'
        + '                <td>\n'
        + '                    <div class="link-button-compat ui-state-default ui-corner-all" style="width: 78px; margin: 0px;"><a href="javascript:void" onclick="app.functionList['+j+'].send(); return false;"><span class="ui-icon ui-icon-signal-diag"></span>Senden</a></div>'
        + '                </td>\n'
        + '                <td style="text-align: center">\n'
        + '                    <div class="link-button-compat ui-state-default ui-corner-all" style="width: 130px; margin: auto;"><a href="javascript:void" onclick="app.functionList['+j+'].blackboard(); return false;"><span class="ui-icon ui-icon-comment"></span>Schwarzes Brett</a></div>'
        + '                </td>\n'
        + '                <td style="text-align: right">\n'
        + '                    <div class="link-button-compat ui-state-default ui-corner-all" style="width: 110px; margin: 0px 0px 0px auto;"><a href="javascript:void" onclick="app.functionList['+j+'].delchat(); return false;"><span class="ui-icon ui-icon-closethick"></span>Chat l&ouml;schen</a></div>'
        + '                </td>\n'
        + '            </tr>\n'
        + '        </table>\n'
        + '    </div>\n'
        + '</div>\n';
    this.blackboard = function() {
        window.open(ajaxsync.baseURL+'blackboard?id='+chatroom,chatroom+'_blackboard','width=600,height=500,left=0,top=0,resizable=yes,scrollbars=yes');
    };
    /**
     * For blinking the chatroom name, toggle the class sigNewMsg on and off ...
     * This is done by a timer created in the handler function and destroyed in
     * the show method.
     */
    this.toggleBlink = function(r) {
        if (r) {
            blinkOn = false;
            $('#link_'+chatroom).removeClass( "sigNewMsg" );
        }
        else {
            if (blinkOn) {
                blinkOn = false;
                $('#link_'+chatroom).removeClass( "sigNewMsg" );
            }
            else {
                blinkOn = true;
                $('#link_'+chatroom).addClass( "sigNewMsg" );
            }
        }
    }
    /**
     * Private method: event handler for displaying messages. Was created
     * in AjaxSync during the chatroom creation or by joining a chatroom
     * during an invite.
     * @param msg Message string received from the server.
     * @function
     */
    function handler(msg) {
        var msgdiv = document.getElementById("msg_" + chatroom);
        var oldTop = msgdiv.scrollTop;
        var oldHeight = msgdiv.scrollHeight;
        var onBottom = (oldTop >= (oldHeight - msgdiv.offsetHeight - 10));
        msgdiv.innerHTML += msg;
        if (onBottom)
            msgdiv.scrollTop = msgdiv.scrollHeight - msgdiv.offsetHeight;
        newmessage = true;
        if (!blinker && hidden) // && newmessage
            blinker = window.setInterval("app.functionList["+j+"].toggleBlink();",500);
	try {
            soundManager.play('ping');
        }
        catch (err) {
            app.displayError('soundmanager2: ' + err.description);
            // replay initialization sequence from soundmanager2.js
            soundManager = new SoundManager();
            soundManager.beginDelayedInit();
            app.displayError('soundmanager2 resetted, create Sound');
            soundManager.onready(function() {
                soundManager.createSound({
                    id: 'ping',
                    url: ajaxsync.baseURL+'sounds/KDE_Event_1.mp3'
                });
            });
            app.displayError('soundmanager2: Sound created');
        }
    }
    /**
     * Public method: Returns the name of the chatroon.
     * @return Name of the chatroon.
     * @function
     */
    this.getName = function() {
        return chatroom;
    }
    /**
     * Public method: signals that this chatroom is hidden. This function
     * is called only from the jQuery tab select event handler.
     * @function
     */
    this.hide = function() {
        hidden = true;
    }
    /**
     * Public method: signals that this chatroom is visible. This function
     * is called only from the jQuery tab select event handler.
     * @function
     */
    this.show = function() {
        hidden = false;
        newmessage = false;
        if (blinker) {
            window.clearInterval(blinker);
            this.toggleBlink(true); // reset blinker to state false
            blinker = false;
        }
    }
    /**
     * Public method: calls the AjaxSync send operation for this chatroom
     * function.
     * @function
     */
    this.send = function() {
        var msg = "";
        // delete message from the input text area
        if (app.isWebKit) {
            msg = '<p>'
                + document.getElementById("send_" + chatroom).value
                + '</p>';
            document.getElementById("send_" + chatroom).value = "";
        }
        else {
            msg = $("#send_" + chatroom).html();
            $("#send_" + chatroom).html("");
        }
        // send message to remote receivers, starting with the username
        var dstr = ts.now();
        ajaxsync.send(
            chatroom,
            '<div class="msgtitle ui-dialog-titlebar ui-widget-header ui-corner-all ui-helper-clearfix"><div class="msgdate">' + dstr
                + '</div>'
                + app.loginName
                + '</div><div class="msgbody">' + msg + '</div>',
            180,
            180
        );
        // display message in our own window, but in a different color
        handler(
            '<div class="mymsgtitle ui-dialog-titlebar ui-corner-all ui-helper-clearfix"><div class="msgdate">' + dstr
            + '</div>'
            + app.loginName
            + '</div><div class="msgbody">' + msg + '</div>'
        );
    };
    this.delchatSuccess = function() {
        app.getFunctionList(); // rebuild chatrooms
    };
    this.delchat = function() {
        ajaxsync.sendConnection.call( // sendConnection is defined after loading ajaxsync.js, not just after init
            ajaxsync.baseURL+"delChat",
            "chatroom=" + escape(chatroom),
            ajaxsync.connectionTimeout, // this value is defined even before init
            app.displayError, // NOT ajaxsync.errorHandler, that is defined only after init
            app.functionList[j].delchatSuccess
        );
    };
    // Register the handler function for receiving calls:
    ajaxsync.registry.addHandler(chatroom,handler);
}

/**
 * @namespace This is the AjaxSync example chat application object. It
 * contains:
 * - the two callback handlers for displaying error messages and
 *   showing the login dialog,
 * - a function for performing a users login bound to the login button,
 * - a function for sending a chat message bound to the send button,
 * - and the handler function which will be called be AjaxSync for
 *   displaying a received chat message.
 */
var app = {
    loginName: '',
    functionList: Array(),
    initializing: true,
    ts: new chatTimestamp(),
    displayError: function(e) {
        // escape HTML tags
        e = e.replace("/&/","&amp;");
        e = e.replace("/</","&lt;");
        e = e.replace("/>/","&gt;");
        // show the error message in its own div element
        document.getElementById("error").innerHTML += app.ts.now() + ' ' + e + "<br/>";
    },
    displayRegError: function(e) {
        // escape HTML tags
        e = e.replace("/&/","&amp;");
        e = e.replace("/</","&lt;");
        e = e.replace("/>/","&gt;");
        // show the error message in its own div element
        document.getElementById("regerror").innerHTML = e;
    },
    autoLogin: function() {
        ajaxsync.login(app.loginName, '');
    },
    logout: function() {
        ajaxsync.logout(); // stop event loop, re-initializes objects
        document.getElementById("regerror").innerHTML = ""; // start with empty error output
        document.getElementById("email").value = '';
        document.getElementById("username").value = '';
        document.getElementById("passwd1").value = '';
        document.getElementById("passwd2").value = '';
        document.getElementById("chatwidget").style.display = "none";
        document.getElementById("login").style.display = "block"; // hide chat, bring up login mask
        app.initializing = true; // make sure that a re-login starts with initializing the application
        ajaxsync.sendConnection.call(
            ajaxsync.baseURL+"logout", // send logout header to browser (reset digest credentials / clear cache)
            "",
            ajaxsync.connectionTimeout, // this value is defined even before init
            app.displayRegError, // NOT ajaxsync.errorHandler, that is defined only after init
            app.registerSuccess
        );
    },
    registerSuccess: function() {
        document.getElementById("regerror").innerHTML
            = app.registrationSuccess;
        document.getElementById("email").value = '';
        document.getElementById("username").value = '';
        document.getElementById("passwd1").value = '';
        document.getElementById("passwd2").value = '';
    },
    clickRegister: function() {
        email = document.getElementById("email").value;
        username = document.getElementById("username").value;
        pw1 = document.getElementById("passwd1").value;
        pw2 = document.getElementById("passwd2").value;
        // standard Ajax call, made using the AjaxSync sendConnection -
        // using this object ensures that the standard Ajax calls are
        // not interfering with the AjaxSync send calls.
        document.getElementById("regerror").innerHTML = app.loadtext;
        ajaxsync.sendConnection.call( // sendConnection is defined after loading ajaxsync.js, not just after init
            ajaxsync.baseURL+"registerMail",
            "email=" + escape(email) + "&name=" + escape(username) + "&pw1=" + escape(pw1) + "&pw2=" + escape(pw2),
            ajaxsync.connectionTimeout, // this value is defined even before init
            app.displayRegError, // NOT ajaxsync.errorHandler, that is defined only after init
            app.registerSuccess
        );
    },
    /**
     * Answer to the call of #getFunctionList.
     *
     * Gets the username and list of readable function from the server even
     * before the AjaxSync login.
     * @param msg Message string received from the server.
     * @function
     */
    storeFunctionList: function(msg) {
        $('#inviteuser').autocomplete( "destroy" );
        $('#chatwidget').tabs( "destroy" );
        $('#editarea').tabs( "destroy" );
        // destroy all previously registered chatrooms
        $("#inviteuser").autocomplete({
            source: function(request,response) {
                function cb(data) {
                    response(jQuery.parseJSON(data));
                }
                ajaxsync.sendConnection.call(
                    ajaxsync.baseURL+"searchuser",
                    "term="+request.term,
                    ajaxsync.connectionTimeout,
                    ajaxsync.errorHandler,
                    cb
                );
            },
            minLength: 0
        });
        $('#chatwidget').tabs({
            select: function(e,ui) {
                function cb(data) {
                    var answer = '<table cellspacing="0">';
                    var list = jQuery.parseJSON(data);
                    for(var i = 0; i < list.length; i++) {
                        answer += '<tr><td>'+list[i].name+'</td><td><img src="'+ajaxsync.baseURL+'images/'+list[i].online+'.gif" alt=""/></td></tr>';
                    }
                    answer += '</table>';
                    document.getElementById('onlinetable').innerHTML = answer;
                }
                if (ui.panel.id == 'online') {
                    ajaxsync.sendConnection.call(
                        ajaxsync.baseURL+"friends",
                        "",
                        ajaxsync.connectionTimeout,
                        ajaxsync.errorHandler,
                        cb
                    );
                }
                else if (ui.panel.id == 'logout') {
                    app.logout();
                    return false;
                }
            }
        });
        document.getElementById("editarea").innerHTML = "";
        ajaxsync.registry.clearHandlers();
        app.functionList = Array();
        app.functionIdx = Object(); // index object, if somebody wants to send data to a chatroom object
        // start parsing server answer
        var msglist = msg.split("\n");
        var roomtabs = "";
        var selectroom = "<select><option selected>-</option>";
        if (msglist.length) {
            // store the username outside the dialog
            app.loginName = msglist[0];
            app.isWebKit = (msglist[1] == "1" ? true : false);
            var conflist = new Object();
            var j = 0;
            for (var i = 2; i < msglist.length; i++) {
                var msgline = msglist[i];
                if (msgline) {
                    var call = ajaxsyncDataExchange.readList(msgline);
                    app.functionIdx[call.fname] = j;
                    app.functionList[j]
                        = new chatroomObject(j,call.fname,call.isowned);
                    roomtabs += '<li><a href="#tab_' + call.fname
                            + '" id="link_' + call.fname + '">' + call.fname + '</a></li>\n';
                    if (call.isowned == 1)
                        selectroom += "<option>" + call.fname + "</option>";
                    j++;
                }
            }
        }
        selectroom += "</select>";
        document.getElementById("inviteroom").innerHTML += selectroom;
        document.getElementById("editarea").innerHTML += "<ul id='roomtabs'>" + roomtabs + "</ul>";
        $('#editarea').tabs({
            select: function(e,ui) {
                var chatroom = ui.panel.id.replace(/^tab_/,'');
                var idx = app.functionIdx[chatroom];
                for (var i = 0; i < app.functionList.length; i++)
                    if (i != idx) app.functionList[i].hide();
                app.functionList[idx].show();
            }
        });
        $('#editarea')
            .addClass("tabs-bottom")
            .removeClass( "ui-tabs-panel" );
        $( ".tabs-bottom > *" )
            .removeClass( "ui-tabs-nav" )
            .addClass("ui-tabs-nav-bottom");
        $( ".ui-tabs-nav-bottom > *" )
            .removeClass( "ui-tabs-panel ui-corner-all ui-corner-top" )
            .addClass( "ui-corner-bottom" );
        $( ".chatTab, .chatOutput > *" )
            .removeClass( "ui-tabs-panel ui-widget-content ui-corner-bottom" );
        if (!app.isWebKit) {
            $('textarea.tinymce').tinymce({
                // Location of TinyMCE script
                script_url : ajaxsync.baseURL+'js/tiny_mce/tiny_mce.js',

                // General options
                theme : "advanced",
                plugins : "pagebreak,style,layer,table,save,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,preview,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template,advlist",

                // Theme options
                theme_advanced_buttons1 : "bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,formatselect,fontselect,fontsizeselect",
                theme_advanced_buttons2 : "cut,copy,paste,pastetext,pasteword,|,bullist,numlist,|,outdent,indent,blockquote,|,undo,redo,|,link,unlink,image,help",
                theme_advanced_buttons3 : "forecolor,backcolor,|,charmap,emotions,iespell,",
                theme_advanced_toolbar_location : "top",
                theme_advanced_toolbar_align : "left",
                theme_advanced_statusbar_location : "bottom",

                // Example content CSS (should be your site CSS)
                content_css : ajaxsync.baseURL+"css/style.css",

                // replacing the return key:
                setup : function (ed) {
                    var idxstr = ed.id.replace(/^send_/,'');
                    var idx = app.functionIdx[idxstr];
                    app.functionList[idx].editor = ed;
                    ed.onKeyPress.addToTop(
                        function (ed, evt) {
                            if ((evt.charCode == 13 || evt.keyCode == 13)) {
                                app.functionList[idx].send();
                                window.setTimeout("app.functionList[" + idx + "].editor.setContent('');",10);
                                return false;
                            }
                        }
                    );
                }
            });
        }
        else {
            $('textarea.tinymce').keypress(function(event){
                if (event.which == 13) {
                    var chatroom = $(this).attr('id').replace(/^send_/,'');
                    var idx = app.functionIdx[chatroom];
                    app.functionList[idx].send();
                    return false;
                }
            });
        }
        if (app.initializing) {
            app.initializing = false;
            ajaxsync.init(app.displayError, app.autoLogin);
            document.getElementById("error").innerHTML = ""; // start with empty error output
            document.getElementById("login").style.display = "none";
            document.getElementById("chatwidget").style.display = "block";
            $('#chatwidget').tabs('select','#editarea');
            if(app.functionList.length) {
                $('#editarea').tabs('select','#tab_'+app.functionList[0].getName());
                app.functionList[0].show(); // ???? not called by select???
            }
        }
        else {
            ajaxsync.login(app.loginName, ''); // re-register handlers
        }
    },
    /**
     * Gets the username and list of readable function from the server even
     * before the AjaxSync login.
     * @function
     */
    getFunctionList: function() {
        ajaxsync.sendConnection.call(ajaxsync.baseURL+"userinfo",
            "",
            ajaxsync.connectionTimeout,
            ajaxsync.errorHandler,
            this.storeFunctionList);
    },
    inviteError: function(e) {
        // escape HTML tags
        e = e.replace("/&/","&amp;");
        e = e.replace("/</","&lt;");
        e = e.replace("/>/","&gt;");
        // show the error message in its own div element
        document.getElementById('inviteresponse').innerHTML = e;
    },
    clickInviteResponse: function(data) {
        document.getElementById('inviteresponse').innerHTML
            = app.invitationSuccess;
        document.getElementById('inviteuser').value = '';
        document.getElementById('inviteroomname').value = '';
        app.getFunctionList();
    },
    clickInvite: function() {
        var user = document.getElementById('inviteuser').value;
        var room = document.getElementById('inviteroom').firstChild.value;
        var name = document.getElementById('inviteroomname').value;
        document.getElementById('inviteresponse').innerHTML
            = app.loadtext;
        ajaxsync.sendConnection.call(ajaxsync.baseURL+"invite",
            "user=" + escape(user) + "&room=" + escape(room) + "&name=" + escape(name),
            ajaxsync.connectionTimeout,
            this.inviteError,
            this.clickInviteResponse);
    },
    clickPWChangeResponse: function(msg) {
        document.getElementById('pw1').value = '';
        document.getElementById('pw2').value = '';
        document.getElementById('pwchangeanswer').innerHTML = msg;
    },
    clickPWChange: function() {
        var pw1 = document.getElementById('pw1').value;
        var pw2 = document.getElementById('pw2').value;
        ajaxsync.sendConnection.call(ajaxsync.baseURL+"changePassword",
            "pw1=" + escape(pw1) + "&pw2=" + escape(pw2),
            ajaxsync.connectionTimeout,
            app.displayError,
            this.clickPWChangeResponse
        );
    }
};
